'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.base = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _index = require('../connection-group/index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var hooks = {
  read: {
    node: [],
    field: []
  },
  write: [],
  update: [],
  request: []
};

/**
 * An empty plugin object. Every plugin group
 * will boil down to one of these.
 * @type {Object}
 */
var base = exports.base = {

  /**
   * Lifecycle hooks.
   * @type {Object}
   */
  hooks: {
    before: hooks,
    after: hooks,
    catch: hooks
  },

  /**
   * A list of storage drivers.
   * @type {Array}
   */
  storage: [],

  /**
   * API extensions.
   * @type {Object}
   */
  extend: {
    root: {},
    context: {}
  },

  /**
   * Query engines.
   * @type {Object}
   */
  engines: {}
};

var merge = {

  /**
   * Merges read hooks together.
   * @param  {Object} hooks - Read hooks.
   * @param  {Function[]} hooks.node - Node-related read events.
   * @param  {Function[]} hooks.field - Field-related read events.
   * @param  {Object} target - Read-related hooks to add.
   * @return {Object} - The merged read hooks.
   */
  read: function read(_ref) {
    var node = _ref.node,
        field = _ref.field;
    var target = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return {
      node: node.concat(target.node || []),
      field: node.concat(target.field || [])
    };
  },

  /**
   * Merges a hook object with another.
   * @param  {Object} hook - A collection of hooks to extend.
   * @param  {Object} hook.read - Read hooks.
   * @param  {Array} hook.write - The base hook to extend.
   * @param  {Array} hook.update - The base hook to extend.
   * @param  {Array} hook.request - The base hook to extend.
   * @param  {Object} target={} - The hooks to add.
   * @return {Object} - The merged hook object.
   */
  hook: function hook(_ref2) {
    var read = _ref2.read,
        write = _ref2.write,
        update = _ref2.update,
        request = _ref2.request;
    var target = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return {
      read: merge.read(read, target.read),
      write: write.concat(target.write || []),
      update: update.concat(target.update || []),
      request: request.concat(target.request || [])
    };
  },

  /**
   * Merges hook objects together.
   * @param  {Object} hooks - Database lifecycle hooks.
   * @param  {Object} hooks.before - Hooks to run before an event.
   * @param  {Object} hooks.after - Hooks to run after an event.
   * @param  {Object} hooks.catch - Hooks to run after an error.
   * @param  {Object} ext={} - Hooks to merge with the base.
   * @return {Object} - The merged hooks object.
   */
  hooks: function hooks(_hooks) {
    var ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return {
      before: merge.hook(_hooks.before, ext.before),
      after: merge.hook(_hooks.after, ext.after),
      catch: merge.hook(_hooks.catch, ext.catch)
    };
  },

  /**
   * Adds a router to the config.
   * @throws {Error} If more than one router was given.
   * @param  {undefined} existing - Should never exist.
   * @param  {Function} router - A router factory.
   * @return {Function} - The router factory.
   */
  router: function router(existing, _router) {
    if (existing && _router) {
      var msg = 'Cannot combine routers. ' + 'Ensure `config.router` is only specified once.';

      throw new Error(msg);
    }

    return _router;
  },

  /**
   * Merges two method objects.
   * @param  {Object} base - The base object to extend.
   * @param  {Object} ext={} - An object with method extensions.
   * @param  {Object} ext.root - Methods to add to a root context.
   * @param  {Object} ext.context - Methods for node contexts.
   * @return {Object} - The combined extensions object.
   */
  extensions: function extensions(_ref3) {
    var root = _ref3.root,
        context = _ref3.context;
    var ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return {
      root: (0, _assign2.default)({}, root, ext.root),
      context: (0, _assign2.default)({}, context, ext.context)
    };
  },

  /**
   * Merges two network configurations together.
   * @param  {ConnectionGroup} [base] - A connection or connection group.
   * @param  {ConnectionGroup|Object} extension - A connection or group to add.
   * @return {ConnectionGroup|Object} - The merged network object.
   */
  network: function network(base, extension) {

    // Support connections not inside a group.
    if (base) {
      base = _index2.default.ensure(base);
    }

    if (extension) {
      extension = _index2.default.ensure(extension);
    }

    // Merge two connection groups together.
    if (base && extension) {
      return base.union(extension);
    }

    return extension;
  },

  /**
   * Merges two storage arrays together.
   * @param  {Array} base - A list of storage interfaces.
   * @param  {Array} ext=[] - A list of storage interfaces.
   * @return {Array} - The merged list.
   */
  storage: function storage(base) {
    var ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    return base.concat(ext);
  },

  /**
   * Merges two engine objects together.
   * @param  {Object} base - Collection of query engines.
   * @param  {Object} ext={} - Collection of query engines.
   * @return {Object} - The merged collection.
   */
  engines: function engines(base) {
    var ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    return (0, _assign2.default)({}, base, ext);
  },

  /**
   * Merges two plugin objects together.
   * @param  {Object} base - The base plugin object to extend.
   * @param  {Object} base.hooks - Database lifecycle hooks.
   * @param  {Array} base.storage - A list of storage drivers.
   * @param  {Object} base.network - A list of network drivers.
   * @param  {Object} base.extend - Method extensions.
   * @param  {Object} ext - An object to merge with the base.
   * @return {Object} - The complete, merged plugin object.
   */
  plugins: function plugins(base, ext) {
    return {
      hooks: merge.hooks(base.hooks, ext.hooks),
      router: merge.router(base.router, ext.router),
      extend: merge.extensions(base.extend, ext.extend),
      storage: merge.storage(base.storage, ext.storage),
      network: merge.network(base.network, ext.network),
      engines: merge.engines(base.engines, ext.engines)
    };
  }
};

/**
 * Merges plugin objects into a single object.
 * @param  {Object[]} plugins - A list of plugin objects.
 * @return {Object} - The merged plugin object.
 */

exports.default = function (plugins) {
  var config = plugins.reduce(merge.plugins, base);

  return (0, _extends3.default)({}, config, {

    // Default the router to "null".
    router: config.router || null,

    // Add a default (empty) network group.
    network: config.network || new _index2.default()
  });
};